/* Internal defenitions for pthreads library.
   Copyright (C) 2000 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#ifndef _PT_INTERNAL_H
#define _PT_INTERNAL_H	1

#include <pthread.h>
#include <stddef.h>
#include <sched.h>
#include <assert.h>

#include <bits/atomic.h>

#include <pt-sysdep.h>
#include <pt-machdep.h>

/* XXX not implemented yet.  */
#define pthread_rwlock_t __pthread_spinlock_t
#define __pthread_rwlock_rlock __pthread_spin_lock
#define __pthread_rwlock_wlock __pthread_spin_lock
#define __pthread_rwlock_unlock __pthread_spin_unlock

/* Thread state.  */
enum pthread_state
{
  PTHREAD_JOINABLE = 0,
  PTHREAD_DETACHED,
  PTHREAD_EXITED,
  PTHREAD_TERMINATED
};

#ifndef PTHREAD_SYSDEP_MEMBERS
# define PTHREAD_SYSDEP_MEMBERS
#endif

/* This structure describes a POSIX thread.  */
struct __pthread
{
  /* Thread ID.  */
  pthread_t thread;

  /* Cancellation.  */
  int cancel_state;
  int cancel_type;
  int cancel_pending;
  struct __pthread_cancelation_handler *cancelation_handlers;

  /* Thread stack.  */
  void *stackaddr;
  size_t stacksize;
  int stack;			/* Nonzero if the stack was allocated.  */

  /* Exit status.  */
  void *status;

  /* Thread state.  */
  enum pthread_state state;
  pthread_mutex_t state_lock;	/* Locks the state.  */
  pthread_cond_t state_cond;	/* Signalled when the state changes.  */

  /* Thread context.  */
  struct pthread_mcontext mcontext;

  PTHREAD_SYSDEP_MEMBERS

  struct __pthread *next, **prevp;
};

/* The total number of threads currently active.  */
extern __atomic_t __pthread_total;

/* The first (main) thread.  */
extern struct __pthread __pthread_main;

/* */
extern struct __pthread **__pthread_threads;
extern pthread_rwlock_t __pthread_threads_lock;

#define __pthread_getid(thread) \
  ({ struct __pthread *__t;                                                  \
     __pthread_rwlock_rlock (&__pthread_threads_lock);                       \
     __t = __pthread_threads[thread];                                        \
     __pthread_rwlock_unlock (&__pthread_threads_lock);                      \
     __t; })

#define __pthread_setid(thread, pthread) \
  __pthread_rwlock_wlock (&__pthread_threads_lock);                          \
  __pthread_threads[thread] = pthread;                                       \
  __pthread_rwlock_unlock (&__pthread_threads_lock);

/* Similar to pthread_self, but returns the thread descriptor instead
   of the thread ID.  */
extern struct __pthread *_pthread_self (void);

/* Save the pthread's self structure.  */
extern inline void __pthread_save_self (struct __pthread *p);



/* Initialize the pthreads library.  */
extern void __pthread_initialize (void);

/* Allocate a new thread structure and its pthread thread ID (but not
   a kernel thread).  */
extern int __pthread_alloc (struct __pthread **thread);

/* Deallocate THREAD.  This is the dual of __pthread_alloc (i.e. it
   does not deallocate the stack, nor does it __pthread_halt the
   thread).  */
extern void __pthread_dealloc (struct __pthread *thread);

extern int __pthread_stack_alloc (void **stackaddr, size_t stacksize);
extern void __pthread_stack_dealloc (void *stackaddr, size_t stacksize);

extern int __pthread_setup (struct __pthread *thread,
			    void (*entry_point)(void *(*)(void *), void *),
			    void *(*start_routine)(void *), void *arg);

/* Start THREAD.  */
extern int __pthread_start (struct __pthread *thread);

#ifndef __pthread_halt
/* Stop THREAD.  */
extern void __pthread_halt (struct __pthread *thread);
#endif

/* Block THREAD.  */
extern void __pthread_block (struct __pthread *thread);

/* Wakeup THREAD.  */
extern void __pthread_wakeup (struct __pthread *thread);

/* Perform a cancelation.  */
extern int __pthread_do_cancel (struct __pthread *thread);


/* Thread attributes.  */

/* This structure describes the attributes of a POSIX thread.  Note
   that not all of them are supported on all systems.  */
struct __pthread_attr
{
  struct sched_param schedparam;
  void *stackaddr;
  size_t stacksize;
  enum __pthread_detachstate detachstate;
};

/* Default attributes.  */
extern struct __pthread_attr __pthread_default_attr;

#endif /* pt-internal.h */
