/* Mutex type.  Generic version.
   Copyright (C) 2000 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#ifndef _BITS_MUTEX_H

#ifndef __need_pthread_mutex
# define _BITS_MUTEX_H	1
#endif

#ifndef __pthread_mutex_defined
# if defined __need_pthread_mutex || defined _BITS_MUTEX_H
#  undef __need_pthread_mutex
#  define __pthread_mutex_defined

#  include <bits/spin-lock.h>

/* User visible part of a mutex.  */
struct __pthread_mutex
  {
    __pthread_spinlock_t __held;
    __pthread_spinlock_t __lock;
    struct __pthread *__queue;
    struct __pthread_mutexattr *__attr;
    void *__data;
  };

/* Initializer for a mutex.  */
#  define __PTHREAD_MUTEX_INITIALIZER \
    ((struct __pthread_mutex) \
      { __SPIN_LOCK_INITIALIZER, __SPIN_LOCK_INITIALIZER, NULL, NULL, NULL })

# endif
#endif /* Not __pthread_mutex_defined.  */

#ifdef _BITS_MUTEX_H

#include <errno.h>
#include <stddef.h>

#ifdef __USE_EXTERN_INLINES

# ifndef _EXTERN_INLINE
#  define _EXTERN_INLINE extern __inline
# endif

/* XXX */
typedef int pthread_mutexattr_t;

_EXTERN_INLINE int
pthread_mutex_init (struct __pthread_mutex *__mutex,
		    const pthread_mutexattr_t *attr)
{
  *__mutex = __PTHREAD_MUTEX_INITIALIZER;
  return 0;
}

_EXTERN_INLINE int
pthread_mutex_destroy (struct __pthread_mutex *__mutex)
{
  return 0;
}

_EXTERN_INLINE int
__pthread_mutex_lock (struct __pthread_mutex *__mutex)
{
  extern int _pthread_mutex_lock (struct __pthread_mutex *);
  
  if (__mutex->__attr == NULL
      && __pthread_spin_trylock (&__mutex->__held) == 0)
    return 0;

  return _pthread_mutex_lock (__mutex);
}

_EXTERN_INLINE int
__pthread_mutex_trylock (struct __pthread_mutex *__mutex)
{
  extern int _pthread_mutex_trylock (struct __pthread_mutex *);
  
  if (__mutex->__attr == NULL)
    return __pthread_spin_trylock (&__mutex->__held);

  return _pthread_mutex_trylock (__mutex);
}

extern inline int
pthread_mutex_lock (struct __pthread_mutex *__mutex)
{
  return __pthread_mutex_lock (__mutex);
}

extern inline int
pthread_mutex_trylock (struct __pthread_mutex *__mutex)
{
  return __pthread_mutex_trylock (__mutex);
}

#endif /* Use extern inlines.  */

#endif

#endif /* bits/mutex.h */
