/* Lock a mutex.  Generic version.
   Copyright (C) 2000 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <pthread.h>

#include <pt-internal.h>

/* Lock MUTEX, block if we can't get it.  */
int
__pthread_mutex_lock (struct __pthread_mutex *mutex)
{
  struct __pthread *self = _pthread_self ();

  __pthread_spin_lock (&mutex->__lock);
  if (__pthread_spin_trylock (&mutex->__held) == 0)
    {
      __pthread_spin_unlock (&mutex->__lock);
      return 0;
    }

  /* Add ourselves to the queue.  */
  self->next = mutex->__queue;
  self->prevp = &mutex->__queue;
  if (mutex->__queue)
    mutex->__queue->prevp = &self->next;
  mutex->__queue = self;
  __pthread_spin_unlock (&mutex->__lock);

  /* XXX: There is a race here.  We are on the queue, however, we are
     not yet waiting!  */

  /* Block the thread.  */
  __pthread_block (self);

  return 0;
}

weak_alias (__pthread_mutex_lock, _pthread_mutex_lock);
weak_alias (__pthread_mutex_lock, pthread_mutex_lock);
