/* Setup thread stack.  Hurd/i386 version.
   Copyright (C) 2000 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <stdint.h>

#include <pt-internal.h>

/* The stack layout used on the i386 is:

    -----------------
   |  ARG            |
    -----------------
   |  START_ROUTINE  |
    -----------------
   |  0              |
    -----------------
   |                 |
   |  Fast TSD       |
   |                 |
    -----------------

    On the Hurd, We reserve __hurd_threadvar_max `unsigned long int's'
    of (fast) thread-specific data (TSD).  */

/* Set up the stack for THREAD, such that it appears as if
   START_ROUTINE and ARG were passed to the new thread's entry-point.
   Return the stack pointer for the new thread.  */
static void *
stack_setup (struct __pthread *thread,
	     void *(*start_routine)(void *), void *arg)
{
  uintptr_t *top;

  /* Calculate top of the new stack.  */
  top = (uintptr_t *) ((uintptr_t) thread->stackaddr + thread->stacksize);
  top -= __hurd_threadvar_max;

  /* We cannot use __hurd_threadvar_location_from_sp here since
     __hurd_threadvar_stack_mask and __hurd_threadvar_stack_offset
     might not be initialized yet.  */
  top[_HURD_THREADVAR_THREAD] = (uintptr_t) thread;

  if (start_routine)
    {
      /* Set up call frame.  */
      *--top = (uintptr_t) arg;	/* Argument to START_ROUTINE.  */
      *--top = (uintptr_t) start_routine;
      *--top = 0;		/* Fake return address.  */
    }

  return top;
}

int
__pthread_setup (struct __pthread *thread,
		 void (*entry_point)(void *(*)(void *), void *),
		 void *(*start_routine)(void *), void *arg)
{
  thread->mcontext.pc = entry_point;
  thread->mcontext.sp = stack_setup (thread, start_routine, arg);

  return 0;
}
