/* Start thread.  Mach version.
   Copyright (C) 2000 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <assert.h>
#include <errno.h>
#include <string.h>

#include <mach.h>

#include <pt-internal.h>

/* Prepare a wakeup message.  */
static error_t
create_wakeupmsg (struct __pthread *thread)
{
  kern_return_t err;

  /* Build wakeup message.  */
  thread->wakeupmsg.msgh_bits = MACH_MSGH_BITS (MACH_MSG_TYPE_COPY_SEND, 0);
  thread->wakeupmsg.msgh_size = 0;
  thread->wakeupmsg.msgh_remote_port = __mach_reply_port ();
  thread->wakeupmsg.msgh_local_port = MACH_PORT_NULL;
  thread->wakeupmsg.msgh_seqno = 0;
  thread->wakeupmsg.msgh_id = 0;

  err = __mach_port_insert_right (__mach_task_self (),
				  thread->wakeupmsg.msgh_remote_port,
				  thread->wakeupmsg.msgh_remote_port,
				  MACH_MSG_TYPE_MAKE_SEND);
  if (err)
    return EAGAIN;

  return 0;
}

/* Start THREAD.  We allocate all system-specific resources, including
   a kernel thread, set it up, and get it running.  */
int
__pthread_start (struct __pthread *thread)
{
  error_t err;

  err = create_wakeupmsg (thread);
  if (err)
    return err;

  /* The main thread is already running of course.  */
  if (thread != &__pthread_main)
    {
      err = __thread_create (__mach_task_self (), &thread->kernel_thread);
      if (err)
	return EAGAIN;

      err = __thread_set_pcsp (thread->kernel_thread,
			       1, thread->mcontext.pc,
			       1, thread->mcontext.sp);
      assert_perror (err);

      err = __thread_resume (thread->kernel_thread);
      assert_perror (err);
    }

  return 0;
}
